var Structr = require('structr'),
bindable = require('../bindable'),
Janitor = require('sk/core/garbage').Janitor;



module.exports = Structr({
	
	/**
	 */

	 '__construct': function(doc)
	 {
	 	this._j = new Janitor();
	 	this._schema = {};
	 	this._createSchema();
	 	this._set(doc);
	 },


	/**
	 * syncs the model from the server
	 */

	'sync': function(router)
	{
		if(this._listener) return this;

		this.load();
		this._listener = this.router.on(this._channel(), { type: 'push' }, this.getMethod('_onResponse'));

		return this;
	},

	/**
	 */

	'unsync': function()
	{
		if(this._listener)
		{
			this._listener.dispose();
			this._listener = null;
		}	

		return this;
	},

	/**
	 */

	'load': function()
	{
		this._pull(undefined, null, this.getMethod('_onResponse'))	
	},

	/**
	 */

	'save': function(doc)
	{
		this._doc(doc);
		this._pull('PUT', this);
		return this;
	},

	/**
	 */
	
	'destroy': function()
	{
		this._pull('DELETE');
		return this;
	},

	/**
	 */

	'dispose': function()
	{
		this._janitor.dispose();
		return this;
	},


	/**
	 */

	'_channel': function()
	{
		return this.channel;
	},

	/**
	 */

	'_onResponse': function(response)
	{
		if(response.result) this._set(response.result);
	},

	/**
	 */

	'_set': function(doc)
	{
		this._doc = doc;

		if(!doc) return;

		if(typeof doc == 'object')
		for(var property in doc)
		{
			if(!this[property])
			{
				this[property] = this._bindable(property, doc[property]);
			}
			else
			{
				this[property](doc[property]);
			}
		}	
	},


	/**
	 */

	'_createSchema': function()
	{
		var schema = this.__construct.schema;

		for(var property in schema)
		{
			this[property] = this._bindable(property, undefined, schema[property]);
		}
	},


	/**
	 */

	'_bindable': function(property, value, model)
	{
		this._schema[property] = value;

		var bo = bindable.object(value, model);
		this._j.addDisposable(bo);
		return bo;
	},

	/**
	 */

	'_pull': function(method, data, onResponse)
	{
		this.router.pull(this._channel(), data, { meta: { method: method }}, onResponse || function(){});
	},

	/**
	 */

	'toJSON': function()
	{
		if(typeof this._doc == 'object')
		{
			var doc = {};
			for(var property in this._schema)
			{
				doc[property] = this[property]();
			}
		}
		else
		{
			doc = this._doc;
		}

		return doc;
	}
});




